/*
 * Copyright 1991, 1992, John F. Haugh II
 * All rights reserved.
 *
 * Permission is granted to copy and create derivative works for any
 * non-commercial purpose, provided this copyright notice is preserved
 * in all copies of source code, or included in human readable form
 * and conspicuously displayed on all copies of object code or
 * distribution media.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include "is_types.h"

#ifndef lint
static	char	sccsid[] = "@(#)is_alloc.c	1.4	10:31:59	2/26/92";
#endif

extern	int	db_errno;

#ifdef	NO_EXTEND
isam_extend (file, offset)
file_t	*file;
off_t	offset;
{
	struct	stat	stat;
	long	cursize;
	int	cnt;
	char	zeros[512];

	if (fstat (file->f_key_fd, &stat))
		return (db_errno = DB_EIO), -1;

#ifdef	BSD
	bzero (zeros, sizeof zeros);
#else
	memset (zeros, 0, sizeof zeros);
#endif
	for (cursize = stat.st_size;cursize < offset;) {
		if (cursize & ~(sizeof zeros))
			cnt = sizeof zeros - (stat.st_size & ~(sizeof zeros));
		else if (offset - cursize < sizeof zeros)
			cnt = offset - cursize;
		else
			cnt = sizeof zeros;

		if (write (file->f_key_fd, zeros, cnt) != cnt)
			return (db_errno = DB_EIO), -1;

		cursize += cnt;
	}
	return 0;
}
#endif

off_t
isam_allocate (file)
file_t	*file;
{
	off_t	free = -1,
		new_free = -1;
	struct	stat	stat;

	if (lseek (file->f_key_fd, 0L, 0) == -1)
		return (db_errno = DB_EIO), -1;

	if (read (file->f_key_fd, &free, sizeof free) != sizeof free)
		return (db_errno = DB_EIO), -1;

	if (free == 0) {
		if (fstat (file->f_key_fd, &stat))
			return (db_errno = DB_EIO), -1;

		free = (stat.st_size + file->f_bsize - 1) / file->f_bsize;
#ifdef	NO_EXTEND
		if (isam_extend (file, free * file->f_bsize))
			return -1;
#endif
		if (lseek (file->f_key_fd,
				free * file->f_bsize, 0) == -1)
			return (db_errno = DB_EIO), -1;

		new_free = 0;

		if (write (file->f_key_fd, &new_free,
				sizeof (off_t)) != sizeof (off_t))
			return (db_errno = DB_EIO), -1;

		return free;
	}
	if (lseek (file->f_key_fd, free * file->f_bsize, 0) == -1)
		return (db_errno = DB_EIO), -1;

	switch (read (file->f_key_fd, &new_free, sizeof new_free)) {
		case 0:
			new_free = 0;
			break;
		case sizeof new_free:
			break;
		default:
			return (db_errno = DB_EIO), -1;
	}
	if (lseek (file->f_key_fd, 0L, 0) == -1)
		return (db_errno = DB_EIO), -1;

	if (write (file->f_key_fd, &new_free,
			sizeof new_free) != sizeof new_free)
		return (db_errno = DB_EIO), -1;

	return free;
}

/*
 * isam_free - return an index node to the free list
 */

off_t
isam_free (file, block)
file_t	*file;
off_t	block;
{
	off_t	old_free;
	keyblk_t zero;

	if (block < MAXINDEXFIELDS) {

		/*
		 * Make absolutely certain that no danglers below the
		 * root node are permitted to exist.
		 */

		memset (&zero, 0, file->f_bsize);

		if (lseek (file->f_key_fd, 0L, 0) == -1)
			return (db_errno = DB_EIO), -1;

		if (write (file->f_key_fd, &zero, file->f_bsize) !=
				file->f_bsize)
			return (db_errno = DB_EIO), -1;

		/*
		 * That's it - the root nodes don't go on the free list.
		 * They are free merely by being empty.
		 */

		return 0;
	}

	/*
	 * This is not a root node, so it must be added to the index
	 * list.  Start by adding the head of the free list to the
	 * to-be-freed block.
	 */

	if (lseek (file->f_key_fd, 0L, 0) == -1)
		return (db_errno = DB_EIO), -1;

	if (read (file->f_key_fd, &old_free, sizeof (off_t)) != sizeof (off_t))
		return (db_errno = DB_EIO), -1;

	if (lseek (file->f_key_fd, block * file->f_bsize, 0) == -1)
		return (db_errno = DB_EIO), -1;

	if (write (file->f_key_fd, &old_free, sizeof (off_t)) != sizeof (off_t))
		return (db_errno = DB_EIO), -1;

	/*
	 * The old free list is now appended off the block to be freed.
	 * If this next seek/write pair fails, the list remains consistent,
	 * but the block has been lost.
	 */

	if (lseek (file->f_key_fd, 0L, 0) == -1)
		return (db_errno = DB_EIO), -1;

	if (write (file->f_key_fd, &block, sizeof (off_t)) != sizeof (off_t))
		return (db_errno = DB_EIO), -1;

	return 0;
}
