/*
 * test 10
 *	verify error codes
 *
 *	@(#)test_10.c	1.1	11:55:05	1/29/92
 */

#include <sys/types.h>
#include <stdio.h>
#include <fcntl.h>
#include "is_types.h"
#include "isam.h"

extern	int	db_errno;

fail (test)
int	test;
{
	fprintf (stderr, "failed test 10.%02d\n", test);
	db_error ("error");
	exit (1);
}

main ()
{
	dbm_t	*dbm;
	int	filenum;
	int	i;
	int	rc;
	struct	{
		int	field11;
		int	field12;
	} record;

	/*
	 * Test db_open() return codes
	 */

	if (dbm = db_open ("test9", O_RDWR))
		fail (1);		/* shouldn't exist */

	if (db_errno != DB_NOT_FOUND)
		fail (2);		/* required error message */

	if (dbm = db_open ("test0", 12345))
		fail (3);		/* invalid open mode */

	if (db_errno != DB_INVALID)
		fail (4);		/* required error message */

	if (dbm = db_open ("test0", O_RDWR|O_CREAT))
		fail (5);		/* database exists already */

	if (db_errno != DB_FILE_EXISTS)
		fail (6);		/* required error message */

	if (! (dbm = db_open ("test0", O_RDWR)))
		fail (7);		/* should exist */

	/*
	 * Test db_open_file() return codes
	 */

	unlink ("test2.idx");
	unlink ("test2.dat");

	if (db_open_file (dbm, "test9") != -1)
		fail (11);		/* file does not exist */

	if (db_errno != DB_NO_FILE)
		fail (12);		/* required error message */

	if (db_open_file (dbm, "test2") != -1)
		fail (13);		/* physical files are gone */

	if (db_errno != DB_NOT_FOUND)
		fail (14);		/* required error message */

	if ((filenum = db_open_file (dbm, "test1")) == -1)
		fail (15);		/* should exist */

	/*
	 * Test db_open_index() return codes
	 */

	if (db_open_index (dbm, -1, "test") != -1)
		fail (21);		/* invalid argument */

	if (db_errno != DB_INVALID)
		fail (22);		/* required error message */

	if (db_open_index (dbm, 1, "test") != -1)
		fail (23);		/* file not open */

	if (db_errno != DB_NOTOPEN)
		fail (24);		/* required error message */

	if (db_open_index (dbm, filenum, "test") != -1)
		fail (25);		/* no such index field */

	if (db_errno != DB_NO_FIELD)
		fail (26);		/* required error message */

	if (db_open_index (dbm, filenum, "field11") != -1)
		fail (27);		/* not an index field */

	if (db_errno != DB_NO_FIELD)
		fail (28);		/* required error message */

	if (db_open_index (dbm, filenum, "index1") == -1)
		fail (29);

	/*
	 * Test db_read_file return codes
	 */

	if (db_rewind (dbm, filenum))
		fail (30);

	for (i = 0;i < 10;i++) {
		rc = db_read_record (dbm, filenum, (void *) &i, IS_EQUAL);

		if (i == 0 || i > 5) {
			if (rc != -1 || db_errno != DB_NO_MATCH)
				fail (31);	/* records don't exist */
		} else {
			if (rc != 0)
				fail (32);	/* records do exist */
		}
	}
	if (db_rewind (dbm, filenum))
		fail (33);

	i = 0;
	while (db_read_record (dbm, filenum, (void *) 0, IS_NEXT) == 0)
		i++;

	if (i != 5)
		fail (34);	/* should be 5 records */

	if (db_errno != DB_EOF)
		fail (35);	/* should be end of file */

	i = 0;
	while (db_read_record (dbm, filenum, (void *) 0, IS_PREV) == 0)
		i++;

	if (i != 4)
		fail (36);	/* should only be 4 records */

	if (db_errno != DB_BOF)
		fail (37);	/* should be beginning of file */

	/*
	 * Test db_write_record return codes
	 */

	for (i = 0;i < 10;i++) {
		record.field11 = i;
		record.field12 = 10 - i;

		rc = db_write_record (dbm, filenum, &record);

		if (i == 0 || i > 5) {
			if (rc != 0)
				fail (41);
		} else {
			if (rc != -1 || db_errno != DB_EXISTS)
				fail (42);
		}
	}

	/*
	 * Test db_update_record
	 */

	if (db_update_record (dbm, -1, &record) != -1)
		fail (51);

	if (db_errno != DB_INVALID)
		fail (52);

	if (db_update_record (dbm, 1, &record) != -1)
		fail (53);

	if (db_errno != DB_NOTOPEN)
		fail (54);

	if (db_rewind (dbm, filenum))
		fail (55);

	if (db_update_record (dbm, filenum, &record) != -1)
		fail (56);

	if (db_errno != DB_NO_RECORD)
		fail (57);

	for (i = 0;i < 10;i++) {
		if (db_read_record (dbm, filenum, &i, IS_EQUAL))
			fail (58);	/* record exists */

		record.field11 = i;
		record.field12 = i * i;

		if (db_update_record (dbm, filenum, &record))
			fail (59);	/* supposed to be OK */
	}

	/*
	 * Test db_delete_file
	 */

	if (db_delete_file (dbm, "test9") != -1)
		fail (81);	/* file does not exist */

	if (db_errno != DB_NO_FILE)
		fail (82);	/* required error message */

	if (db_delete_file (dbm, "test3"))
		fail (83);	/* file does exist */

	if (db_delete_file (dbm, "test2") != -1)
		fail (84);	/* physical files are gone already */

	if (db_errno != DB_NOT_FOUND)
		fail (85);	/* required error message */

	/*
	 * Test db_close_file and db_close return codes
	 */

	if (db_close_file (dbm, 1) != -1)
		fail (91);		/* file wasn't opened */

	if (db_errno != DB_NOTOPEN)
		fail (92);		/* required error message */

	if (db_close_file (dbm, -1) != -1)
		fail (93);		/* invalid argument */

	if (db_errno != DB_INVALID)
		fail (94);		/* required error message */
	
	if (db_close_file (dbm, filenum))
		fail (95);		/* was still open */

	if (db_close (dbm))
		fail (96);		/* was still open */

	exit (0);
}
